local ADDON, Package = ...

local Module = {}
local Registry = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------
local camelcase = Package.String.camelcase
local merge = Package.Enum.merge
local rename = Package.Enum.rename
local create_character_information = Package.Utils.create_character_information

--------------------------------------------------------------------------------
-- Localization
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local PROXY = CreateFrame("Frame")

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.migrate_storage()
  local realm = GetRealmName()
  local storage = MedusaStapleStorage.realm[realm]
  local profession_cooldown = storage.profession_cooldown

  for name, types in pairs(profession_cooldown) do
    local last_spell_id = 0
    local last_spell_timestamp = 0

    for spell_id, spell_timestamp in pairs(profession_cooldown[name]["ALCHEMY"]) do
      last_spell_id = spell_id
      last_spell_timestamp = spell_timestamp
    end

    if last_spell_id > 0 and last_spell_timestamp > 0 then
      profession_cooldown[name]["ALCHEMY"] = {[last_spell_id] = last_spell_timestamp}
    end
  end
end

function Module.load_storage()
  local realm = GetRealmName()
  local player = UnitName("player")

  local x = GetScreenWidth() / 2
  local y = GetScreenHeight() / 2

  local default_schema = {
    version = 2,
    realm = {
      [realm] = {
        use = {
          money = true,
          durability = true,
          raid_information = true,
          move_speed = true,
          location = true,
          profession_cooldown = true
        },
        position = {
          money = {x = x, y = 16 + y},
          durability = {x = x, y = 32 + y},
          raid_information = {x = x, y = 48 + y},
          move_speed = {x = x, y = 64 + y},
          location = {x = x, y = 80 + y},
          profession_cooldown = {x = x, y = 96 + y}
        }
      }
    }
  }

  local player_schema = {
    realm = {
      [realm] = {
        character = {
          [player] = create_character_information()
        },
        money = {
          [player] = {
            amount = 0
          }
        },
        raid_information = {
          [player] = {
            quests = {},
            instances = {}
          }
        },
        location = {
          [player] = {
            zone = "",
            subzone = "",
            position = {x = 0, y = 0},
            hearthstone = {bind = "", timestamp = 0}
          }
        },
        profession_cooldown = {
          [player] = {
            TAILORING = {},
            LEATHERWORKING = {},
            ALCHEMY = {},
            JEWELCRAFTING = {}
          }
        }
      }
    }
  }

  MedusaStapleStorage = merge(MedusaStapleStorage or {}, default_schema, false)
  MedusaStapleStorage = merge(MedusaStapleStorage, player_schema, false)
end

function Module.run()
  PROXY:RegisterEvent("PLAYER_LOGIN")
  PROXY:RegisterEvent("ADDON_LOADED")
  PROXY:SetScript("OnEvent", Module.on_event)
end

function Module.on_event(self, event, ...)
  Registry[event](self, ...)
end

--------------------------------------------------------------------------------
-- Registry
--------------------------------------------------------------------------------
function Registry.PLAYER_LOGIN()
  local realm = GetRealmName()
  local storage = MedusaStapleStorage.realm[realm]
  local use = storage.use

  for option, value in pairs(use) do
    if value then
      Package[camelcase(option)].enable()
    end
  end
end

function Registry.ADDON_LOADED(self, ...)
  if ADDON == ... then
    Module.load_storage()
    Module.migrate_storage()

    local addon = WrapTextInColorCode(..., "ff91d7f2")
    local version = WrapTextInColorCode("v" .. GetAddOnMetadata(..., "Version"), "ff91d7f2")
    local slashes = WrapTextInColorCode(SLASH_MedusaStaple1 .. " " .. SLASH_MedusaStaple2, "ff72f2e5")

    DEFAULT_CHAT_FRAME:AddMessage(addon .. " " .. version .. " " .. slashes )
  end
end

--------------------------------------------------------------------------------
-- Slash Command
--------------------------------------------------------------------------------
SLASH_MedusaStaple1 = "/medusa_staple"
SLASH_MedusaStaple2 = "/ms"
SlashCmdList[ADDON] = function ()
  if not MedusaStapleConfig then
    Package.Config.run()

    table.insert(UISpecialFrames, MedusaStapleConfig:GetName())
  end

  MedusaStapleConfig:SetShown(not MedusaStapleConfig:IsShown())
end

--------------------------------------------------------------------------------
-- Entry Point
--------------------------------------------------------------------------------
Module.run()
