local ADDON, Package = ...

local Module = {}
local Registry = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------
local L = Package.L
local create_label = Package.Widget.create_label

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local PROXY = CreateFrame("Frame")

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------
local function translate_into_percent(amount)
  return math.floor(amount / 7 * 100 + 0.01)
end

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------
local time_since_last = 0
local cache = {
  move = -1
}
local position = {x = 0, y = 0}
local speed = {
  move = -1,
  run = -1,
  flight = -1,
  swim = -1
}

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------
local label = create_label({size = {64, 16}, icon = 134010}, UIParent)

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.on_label_position_change(name, x, y)
  if name == "move_speed" then
    position.x = x
    position.y = y

    label:ClearAllPoints()
    label:SetPoint("BOTTOMLEFT", x, y)
  end
end

function Module.on_label_drag_start(self)
  self:StartMoving()
end

function Module.on_label_drag_stop(self)
  self:StopMovingOrSizing()

  Package.Observable.broadcast("position", "move_speed", self:GetLeft(), self:GetBottom())
end

function Module.on_label_enter(self)
  GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")
  GameTooltip:ClearLines()

  GameTooltip:AddLine(L["move_speed"])
  GameTooltip:AddLine(" ")

  GameTooltip:AddDoubleLine(L["move_speed_run"], translate_into_percent(speed.run) .. "%", 1, 1, 1, 1, 1, 1)
  GameTooltip:AddDoubleLine(L["move_speed_swim"], translate_into_percent(speed.swim) .. "%", 1, 1, 1, 1, 1, 1)
  GameTooltip:AddDoubleLine(L["move_speed_flight"], translate_into_percent(speed.flight) .. "%", 1, 1, 1, 1, 1, 1)

  GameTooltip:Show()
end

function Module.on_label_leave(self)
  GameTooltip:Hide()
end

function Module.update_status()
  speed.move, speed.run, speed.flight, speed.swim = GetUnitSpeed("player")

  if speed.move ~= cache.move then
    label.text:SetFormattedText(translate_into_percent(speed.move) .. "%%")
  end

  cache.move = speed.move
end

function Module.enable()
  local realm = GetRealmName()
  local storage = MedusaStapleStorage.realm[realm]

  position = storage.position.move_speed

  label:RegisterForDrag("LeftButton")
  label:SetScript("OnEnter", Module.on_label_enter)
  label:SetScript("OnLeave", Module.on_label_leave)

  label:SetPoint("BOTTOMLEFT", position.x, position.y)
  label:Show()

  PROXY:SetScript("OnUpdate", Module.on_update)

  Package.Observable.subscribe("position", Module.on_label_position_change)

  Module.update_status()
end

function Module.disable()
  label:Hide()

  PROXY:SetScript("OnUpdate", nil)

  Package.Observable.unsubscribe("position", Module.on_label_position_change)
end

function Module.hot()
  GameTooltip:Hide()

  label:StopMovingOrSizing()
  label:SetScript("OnDragStart", Module.on_label_drag_start)
  label:SetScript("OnDragStop", Module.on_label_drag_stop)
  label:SetScript("OnEnter", nil)
  label:SetScript("OnLeave", nil)

  label.background:Show()
end

function Module.cold()
  GameTooltip:Hide()

  label:StopMovingOrSizing()
  label:SetScript("OnDragStart", nil)
  label:SetScript("OnDragStop", nil)
  label:SetScript("OnEnter", Module.on_label_enter)
  label:SetScript("OnLeave", Module.on_label_leave)

  label.background:Hide()
end

function Module.on_update(self, elapsed)
  time_since_last = time_since_last + elapsed

  if time_since_last > 1.0 then
    time_since_last = 0

    Module.update_status()
  end
end

--------------------------------------------------------------------------------
-- Registry
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Export
--------------------------------------------------------------------------------
Package.MoveSpeed = Module