local ADDON, Package = ...

local MODULE = "RaidPrep"
local STORAGE = "raid_prep"

local create_label = Package.Widget.create_label
local create_buffs = Package.Widget.create_buffs

local create_character_icon = Package.Utils.create_character_icon
local create_character_name = Package.Utils.create_character_name
local seconds_to_time = Package.Utils.seconds_to_time

local namespace = {}
local storage = nil
local object = {
  label = nil
}
local state = {}

function namespace.on_drag_start(self)
  if IsControlKeyDown() then
    self.background:Show()
    self:StartMoving()
    self:SetUserPlaced(false)
  end
end

function namespace.on_drag_stop(self)
  self.background:Hide()
  self:StopMovingOrSizing()

  state.position[STORAGE] = {x = self:GetLeft(), y = self:GetBottom()}
end

function namespace.on_enter(self)
  GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")
  GameTooltip:ClearLines()
  GameTooltip:AddLine("공격대 준비")
  GameTooltip:AddLine(" ")
  GameTooltip_InsertFrame(GameTooltip, object.buffs)

  GameTooltip:Show()
end

function namespace.on_leave(self)
  GameTooltip:Hide()
end

function namespace.update()
  object.label.text:SetText("공격대 준비")
end

function namespace.initialize()
  object.label = create_label({size = {148, 16}, icon = 133784, name = ADDON .. MODULE}, UIParent)
  object.label:RegisterForDrag("LeftButton")
  object.label:SetScript("OnDragStart", namespace.on_drag_start)
  object.label:SetScript("OnDragStop", namespace.on_drag_stop)
  object.label:SetScript("OnEnter", namespace.on_enter)
  object.label:SetScript("OnLeave", namespace.on_leave)
  object.label:SetPoint("BOTTOMLEFT", state.position[STORAGE].x, state.position[STORAGE].y)

  object.buffs = create_buffs(nil, UIParent)
end

function namespace.load()
  local realm = GetRealmName()
  local name = UnitName("player")

  storage = MedusaStapleStorage.realm[realm]

  -- state[STORAGE] = storage[STORAGE][name]
  state.character = storage.character[name]
  state.position = storage.position
end

function namespace.on_event(self, event, ...)
  if namespace[event] then
    namespace[event](self, ...)
  end
end

function namespace.PLAYER_ENTERING_WORLD()
  namespace.update()
end

function namespace.ADDON_LOADED(self, ...)
  if ADDON == ... then
    namespace.load()
    namespace.initialize()
  end
end

Package[STORAGE] = namespace

