local ADDON, Package = ...

local Module = {}
local Registry = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------
local create_label = Package.Widget.create_label

--------------------------------------------------------------------------------
-- Localization
--------------------------------------------------------------------------------
local locale = GetLocale()
local L = Package[locale] or "enUS"

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local PROXY = CreateFrame("Frame")

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------
local cache = {}
local position = {x = 0, y = 0}

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------
local label = create_label({size = {0, 16}, icon = 0}, UIParent)

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.on_label_drag_start(self)
  self:StartMoving()
end

function Module.on_label_drag_stop(self)
  self:StopMovingOrSizing()

  position.x = self:GetLeft()
  position.y = self:GetBottom()
end

function Module.on_label_enter(self)
  GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")
  GameTooltip:ClearLines()
  GameTooltip:Show()
end

function Module.on_label_leave(self)
  GameTooltip:Hide()
end

function Module.enable()
  label:RegisterForDrag("LeftButton")
  label:SetScript("OnEnter", Module.on_label_enter)
  label:SetScript("OnLeave", Module.on_label_leave)

  label:SetPoint("BOTTOMLEFT", position.x, position.y)
  label:Show()

  PROXY:RegisterEvent("")
  PROXY:SetScript("OnEvent", nil)
  PROXY:SetScript("OnUpdate", nil)
end

function Module.disable()
  label:Hide()

  PROXY:UnregisterEvent("")
  PROXY:SetScript("OnEvent", nil)
  PROXY:SetScript("OnUpdate", nil)
end

function Module.hot()
  GameTooltip:Hide()

  label:StopMovingOrSizing()
  label:SetScript("OnDragStart", Module.on_label_drag_start)
  label:SetScript("OnDragStop", Module.on_label_drag_stop)
  label:SetScript("OnEnter", nil)
  label:SetScript("OnLeave", nil)

  label.background:Show()
end

function Module.cold()
  GameTooltip:Hide()

  label:StopMovingOrSizing()
  label:SetScript("OnDragStart", nil)
  label:SetScript("OnDragStop", nil)
  label:SetScript("OnEnter", Module.on_label_enter)
  label:SetScript("OnLeave", Module.on_label_leave)

  label.background:Hide()
end

function Module.on_event(self, event, ...)
  Registry[event](self, ...)
end

function Module.on_update(self, elapsed)
  time_since_last = time_since_last + elapsed

  if time_since_last > 1.0 then
    time_since_last = 0
  end
end

--------------------------------------------------------------------------------
-- Registry
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Export
--------------------------------------------------------------------------------
Package.Module = Module